;;; PLOTCSV.LSP    Ergastirio Topografias Despotakis Vassilios
;;;	-------   ODHGIES -----
;;;	Meta tin exagosi apo Excel  (.xls) se .txt (comma delimited):
;;;	1. Replace ".' se ""
;;;	2. Replace "," se "."
;;;	3. Replace ";" se ","
;;;
;;;	Sto AutoCad: APPLOAD <path>/PLOTCSV.LSP (Enter)
;;;	Command: plotcsv (Enter)
;;;	Height of txt: 0.1
;;;	Altitude, no or Both: B
;;;	Layer Name: PTS
;;;
;;;	Arxizei i apodosi me ta Sym_DTE.LIN Linetypes
;;;	To OSNAP se NODES (on)
;;;	Insert san OLE Objects to EXOFYLLO.DOC kai DILOSI.DOC
;;;	Kanavos
;;;	Sima Borra (Arrowa.dwg)
;;;
;;;	---------------------------------------
;;;      This freeware routine, reads an ASCII file (Comma Separated Values)
;;;      and draws on the current UCS
;;;         a. altitude text-objects of points in a user-defined layer
;;;         b. point-number text-objects in the same or another layer
;;;         c. one node-object for each number-object in the relevant layer
;;;              representing exact X and Y coords
;;;         d. one node-object for each alitude-object in the relevant layer
;;;              also representing exact X,Y coords
;;;
;;;         
;;;         All alt/num/node-objects are drawn co-planar, having elevation Z=0.
;;;         The exact form of such a CSV ASCII file is:
;;;                        Point number,X-coord,Y-coord,H-coord(CRLF)
;;;                        Point number,X-coord,Y-coord,H-coord(CRLF)
;;;                        Point number,X-coord,Y-coord,H-coord(CRLF)
;;;                                        .
;;;                                        .
;;;                                        .
;;;                        Point number,X-coord,Y-coord,H-coord(CRLF)
;;;                        Point number,X-coord,Y-coord,H-coord(CRLF)
;;;                        (EOF)
(defun MODES (a) 
  (setq MLST '
        ())
  (repeat (length a) 
    (setq MLST (append MLST (list (list (car a) (getvar (car a))))))
    (setq a (cdr a))
  )
) 
(defun MODER () 
  (repeat (length MLST) 
    (setvar (caar MLST) (cadar MLST))
    (setq MLST (cdr MLST))
  )
) 
;;; **************************Ascii Text error handler *************************
(defun at_err (st)                    ; If an error (such as CTRL-C) occurs
                                      ; while this command is active...
  (if (and (/= st "Function cancelled") 
           (/= st "quit / exit abort")
      )
    (princ (strcat "\nError: " st))
  ) 
  (moder)                             ; Restore modified modes
  (if (= (type rtfile) 'FILE) 
    (close rtfile)
  ) 
  (setq rtfile nil)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
) 
;;;
;;; **************************** MAIN PROGRAM **********************************
;;;
(Defun C:PLOTCSV (/ fn gr grlen count cnt ycnt zcnt icnt strx stry strz 
		chrht lanm stri i pt ptz pti styn svr
                olderr rtfile rf rfa ); at_fnm holds default FILE
  (setq olderr *error* 
        *error* at_err)
;
;**********************Prompt for file to be inserted***************************
;
  (modes '("BLIPMODE" "CMDECHO" "HIGHLIGHT"))
  (while (null rtfile)                
    (if (null at_fnm)
      (if (= 1 (getvar "FILEDIA"))
        (setq rf (getfiled "Select Text File to Import" "" "" 12))
        (progn
          (initget 1) 
          (princ "\nFile to read (including extension): ")
          (setq rf (getstring))
        );endprogn 
      );endif
      (if (= 1 (getvar "FILEDIA"))
        (setq rf (getfiled "Select Text File to Import" at_fnm "" 12))
        (progn
          (princ "\nFile to read (including extension)/<") 
          (princ (strcat at_fnm ">: "))
          (setq rf (getstring))
        );endprogn
      );endif
    );endif 
    (if (= rf nil) (exit))
    (if (= rf 1)
      (if (null at_fnm)
        (progn
          (initget 1) 
          (princ "\nFile to read (including extension): ")
          (setq rf (getstring))
        );endprogn 
        (progn
          (princ "\nFile to read (including extension)/<") 
          (princ (strcat at_fnm ">: "))
          (setq rf (getstring))
        );endprogn
      );endif
    );endif
    (if (and (= rf "") (/= nil at_fnm)) 
      (setq rf at_fnm)
    );endif 
    (setq rfa (findfile rf))
    (if (= "~" rf) 
      (progn 
        (setq rfa nil)
        (setq rtfile nil)
      );endprogn 
    );endif
    (if rfa 
      (progn
        (setq at_fnm rfa)
        (if (null (setq rtfile (open rfa "r")))
          (princ (strcat 
            "\n\tFile found, but couldn't open " at_fnm " for reading. "))
        );endif
      );endprogn
      (if (/= "~" rf)
        (if (and (< 4 (strlen rf))
                 (/= (substr rf (- (strlen rf) 3) 1) ".")
            );endand
          (princ "\nFile not found.  Extension may be missing.")
          (princ "\nFile not found. ")
        );endif
      );endif
    );endif
  );endwhile 
;
;*************drawing set-up***************************
;
   (setvar "cmdecho" 0)
   (command "undo" "m")      ;if something goes bad then retun here
   (setq svr(getvar "osmode"))
   (setvar "dimzin" 0)
   (setvar "osmode" 0)
   (setvar "unitmode" 0)
   (setvar "aunits" 2)
   (setvar "angbase" (/ pi 2))
   (setvar "angdir" 1)
   (setvar "auprec" 4)
   (setvar "lunits" 2)
   (setvar "luprec" 3)
   (setq count 0) 
   (setq chrht ht)
   (if (null ht)    
       (setq ht (getreal   "\nEnter the height of plotted point-numbers : "))
       (setq ht (getreal   (strcat "\nEnter the height of plotted point-numbers <" (rtos ht 2 2) "> :")))
   )
   (if (null ht)
       (setq ht chrht)
   )
   (initget "Number Altitude Both")
   (setq styn(getkword "Altitude, Number or Both?  : "))
   (setq lanm (getstring "\nLayer-name to plot into: "))
   (command "layer" "m" lanm "")
   (setq txt (open rf "r"))
        (while (setq gr (read-line txt))
	      (setq count (1+ count))
	      (setq cnt 0)
	      (setq grlen (strlen gr))
	    (while (/= (substr gr (+ 1 cnt) 1 ) ",")
		   (setq cnt (1+ cnt))
		   (setq stri (substr gr 1 cnt))
	    );end while
		   (setq ycnt cnt)
		   (setq cnt (+ 2 cnt))
	    (while (/= (substr gr cnt 1 ) ",")
		   (setq cnt (1+ cnt))
		   (setq strx (substr gr (+ 2 ycnt) (- cnt ycnt)))
	    );end while
		   (setq cnt (- cnt 1))
		   (setq zcnt cnt)
		   (setq cnt (+ 2 cnt))
	    (while (/= (substr gr cnt 1 ) ",")
		   (setq cnt (1+ cnt))
		   (setq stry
                                                       (substr gr (+ 2 zcnt) (- cnt (+ 2 zcnt)))
                                        )
	    );end while
		   (setq cnt (- cnt 1))
		   (setq icnt cnt)
		   (setq cnt (+ 2 cnt))
	    (while (<= cnt grlen)
		   (setq cnt (1+ cnt))
		   (setq strz(rtos
                                                               (atof 
                                                                       (substr gr (+ 2 icnt) (- cnt icnt))
                                                               )
                                                        2 3)
                                       )
	    );end while
		   (princ stri)
		   (setq z strz)
		   (setq pt(list (atof strx) (atof stry)))
		   (setq pti(list (atof strx) (- (atof stry) (* ht 1.2))))
		   (setq ptz(list (atof strx) (- (atof stry) (* ht 0.0))))
                   (if(or(= styn "Both")(= styn "Number"))
                      (command "text" "j" "c" pti ht 100 stri)
                   );end if
                   (if(or(= styn "Both")(= styn "Altitude"))
                      (command "text" "j" "c" ptz ht 100 strz)
                   );end if
		   (command "point" pt )
	);end while
    (ok)
);Close defun PLOTCSV
(defun ok ()     
        (setvar "osmode" svr)
	(setvar "cmdecho" 1)
	(close txt)
        (setq txt nil)
        (moder)                             ; Restore modified modes
        (setq *error* olderr)               ; Restore old *error* handler
        (princ)
);close defun ok
